#include "app_gtc.h"
#include "ui_app_gtc.h"
#include "/root/Documents/influxdb/influxdb.hpp"

using namespace std;



app_gtc::app_gtc(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::app_gtc)
{
    ui->setupUi(this); //initialise l'interface utilisateur

    QPalette p = ui->txtDebug->palette();
    p.setColor(QPalette::Base, QColor(240, 240, 255));
    ui->txtDebug->setPalette(p); //colorer l'interface

    ui->le_heure->setText(mHeure.currentTime().toString("hh:mm:ss")); //définition du format de l'heure
    ui->le_date->setText(mDate.currentDate().toString("yyyy-MM-dd")); //définition du format du jour

    ui->cbPort->clear();
    ui->cbPort->addItems(this->listerPorts());  //ajout des ports

    QIntValidator *testInt = new QIntValidator(this);
    testInt->setRange(10,120);
    ui->le_dec->setValidator(testInt);

    QRegExp regex("(([0-1][0-9])|([2][0-3])):([0-5][0-9]):([0-5][0-9])");
    QRegExpValidator *hourValidator = new QRegExpValidator(regex, this);
    ui->le_heure->setValidator(hourValidator);
    ui->le_heure->setInputMask("00:00:00");

}

app_gtc::~app_gtc()
{
    delete ui;
}

void app_gtc::on_pbConnexion_clicked()
{
    if (ui->pbConnexion->text() == "Connexion")
    {
        port = new CSerial(this, "/dev/"+ui->cbPort->currentText());
        connect(port, SIGNAL(sigErreur(QSerialPort::SerialPortError)), this, SLOT(on_erreur(QSerialPort::SerialPortError)));
        connect(port, SIGNAL(sigData(QByteArray)), this, SLOT(on_receiveData(QByteArray)));
        port->initialiser(QSerialPort::Baud9600,
                           QSerialPort::Data8,
                           QSerialPort::NoParity,
                           QSerialPort::OneStop,
                           QSerialPort::NoFlowControl);
        if(port->ouvrirPort())
        {
            ui->pbConnexion->setText("Déconnexion"); //dès que le port est ouvert, afficher le texte "Déconnecter"
            qDebug() << "Port ouvert";
            ui->cbPort->setEnabled(false);
            ui->groupBox->setEnabled(true);
            ui->groupBox_2->setEnabled(true);
        }
        else
        {
            qDebug() << "Le Port n'est pas ouvert !"; //si le port n'est pas ouvert, affiche le message d'erreur
            ui->cbPort->setEnabled(true);
            ui->groupBox->setEnabled(false);
            ui->groupBox_2->setEnabled(false);

        }
    }
    else
    {
        ui->pbConnexion->setText("Connexion");
        ui->cbPort->setEnabled(true);
        ui->groupBox->setEnabled(false);
        ui->groupBox_2->setEnabled(false);

        delete port;
        QString messageFermer = mDate.currentDate().toString("dd/MM/yyyy") + " " + mHeure.currentTime().toString("hh:mm:ss") + " : ";
        messageFermer += "Le port " + ui->cbPort->currentText() + " est fermé";
        ui->txtDebug->append(messageFermer);

    } // else
}

QStringList app_gtc::listerPorts()
{

    QStringList stringList;
    QList<QSerialPortInfo> spi;
    spi = QSerialPortInfo::availablePorts();
    if(!spi.empty())
    {
        for (int i=0 ; i<spi.size() ; i++)
        {
            QString str = spi.at(i).portName();
            stringList.append(str);
        } // for
    }
    return stringList;
}

void app_gtc::on_erreur(QSerialPort::SerialPortError err)
{
    //ui->teDebug->append("Erreur voie série !");
    QString messageErreur = mDate.currentDate().toString("dd/MM/yyyy") + " " + mHeure.currentTime().toString("hh:mm:ss") + " : ";

    qDebug() << "Serial Port : " << err;
    switch (err) {
    case QSerialPort::NoError:
        messageErreur += "Le port " + ui->cbPort->currentText() + " est ouvert";
        break;

    case QSerialPort::DeviceNotFoundError:
        messageErreur += "Le port " + ui->cbPort->currentText() + " n'est pas présent sur le système";
        break;
    case QSerialPort::PermissionError:
        messageErreur += "Permissions d'accéder au port " + ui->cbPort->currentText() + " non accordées";
        break;
    case QSerialPort::OpenError:
        messageErreur += "Erreur lors de l'ouvertur du port " + ui->cbPort->currentText();
        break;
    case QSerialPort::NotOpenError:
        messageErreur += "Le port " + ui->cbPort->currentText() + " est ouvert mais l'opération ne peut être exécutée";
        break;
    case QSerialPort::WriteError:
        messageErreur += "Impossible d'écrire sur le port " + ui->cbPort->currentText();
        break;
    case QSerialPort::ReadError:
        messageErreur += "Impossible de lire le port " + ui->cbPort->currentText();
        break;
    case QSerialPort::ResourceError:
        messageErreur += "Le port " + ui->cbPort->currentText() + " est indisponible. Il a été retiré du système";
        break;

    default:
        messageErreur += "Erreur voie série !";
        break;
    }
    ui->txtDebug->append(messageErreur);
}

char app_gtc::calculChecksum(QByteArray message)
{
    char check = 0;
    for (int i = 0; i < message.length(); i++)
    {
        check = check ^ message.at(i);
    }

    qDebug() << "Checksum = " << check;
    return check;
}


void app_gtc::on_pb_actuDim_clicked()
{
    mOrdre = "DIM";
    mData.setNum(ui->sl_dim->value());

    const char* adresse = mAdresse.constData();
    float donnees = QString(mData).toFloat();   //convertion en flottant
    influxdb_cpp::server_info si("192.168.4.34", 8086, "bddgtc", "", "");
    influxdb_cpp::builder()
            .meas("dim")
            .tag("add", adresse)
            .tag("flag","0")
            .field("value", donnees)
            .post_http(si, &resp);
    this->on_pbRefresh_clicked();
}

void app_gtc::on_pb_actuInt_clicked()
{
    mOrdre = "MAX";
    mData.setNum(ui->sl_int->value());

    const char* adresse = mAdresse.constData();    //convertion de mAdresse en char
    float donnees = QString(mData).toFloat();   //convertion en flottant

    influxdb_cpp::server_info si("192.168.4.34", 8086, "bddgtc", "", "");
    influxdb_cpp::builder()
            .meas("int")
            .tag("add", adresse)
            .tag("flag","0")
            .field("value", donnees)
            .post_http(si, &resp);

    this->on_pbRefresh_clicked();
}

void app_gtc::on_pb_decalage_clicked()
{
    mOrdre = "OFS";
    int offset = ui->le_dec->text().toInt();
    if(offset <10) ui->le_dec->setText("10");
    mData = ui->le_dec->text().toUtf8();

    /*influxdb_cpp::server_info si("192.168.4.34", 8086, "bddgtc", "", "");

    influxdb_cpp::builder()
            .meas("")
            .tag("add","2428")
            .tag("flag","0")
            .field("value", mData)
            .post_http(si,&resp);*/
}

void app_gtc::on_pb_horloge_clicked()
{
    mOrdre = "RTC";

    QString stDate = ui->le_date->text()+"T"+ui->le_heure->text();
    qDebug() << stDate;
    QDateTime laDate = QDateTime::fromString (stDate, Qt::ISODate);
    laDate.setOffsetFromUtc(0);
    qDebug()<< laDate.toString();
    uint timestamp = laDate.toTime_t();
    qDebug() << timestamp;
    mData.setNum(timestamp);

    /*influxdb_cpp::server_info si("192.168.4.34", 8086, "bddgtc", "", "");

    influxdb_cpp::builder()
            .meas("")
            .tag("flag",  "0")
            .tag("add",adresse)
            .field("value", mData)
            .post_http(si,&resp);*/
}

void app_gtc::on_receiveData(QByteArray txt)
{
    qDebug() << txt;
    ui->txtDebug->append("Reçu :");
    ui->txtDebug->append(QString(txt).remove('\n'));

    QList<QByteArray> trame;
    trame = txt.split(SEPARATEUR);
    qDebug() << "Adresse : " << trame.at(0);
    qDebug() << "Ordre : " << trame.at(1);
    qDebug() << "Data : " << trame.at(2);

    if(trame.at(1)=="AMP")
    {
        mAmpere = trame.at(2).toFloat();    //calcul courant
        mWatt = mAmpere*230/1000;   //calcul puissance
        QString sAmpere, sWatt;
        ui->lbCourant->setText(sAmpere.setNum(mAmpere,'f',2)+" mA");
        ui->lbPuissance->setText(sWatt.setNum(mWatt,'f',2)+" W");

            /*influxdb_cpp::server_info si_new("192.168.4.34", 8086, "", "bddgtc", "", "");
            influxdb_cpp::query(resp, "select * from amp",  si_new);
            influxdb_cpp::query(resp, "select * from watt", si_new);*/
    }
}

void app_gtc::on_sb_segment_valueChanged(int arg1)
{
    if(ui->cb_segment->isChecked())
        mAdresse = "0";
    else mAdresse.setNum(arg1);
}

void app_gtc::on_sb_lampadaire_valueChanged(int arg1)
{
    if(ui->cb_segment->isChecked())
        mAdresse = "0";
    else mAdresse.setNum(arg1);
}

void app_gtc::on_cb_segment_stateChanged(int arg1)
{
    qDebug() << "Diffusion : " << arg1;
    if(!arg1)
    {
        mAdresse.setNum(ui->sb_lampadaire->value());
    }
    else mAdresse = "0";
}


/* CONFIGURATION XBEE*/
/*
void app_gtc::slotStatusOK()
{
qDebug() << "Status : OK - Action : " << port->getAction(); //affiche dans le debugger l'action effectuée
ui->txtDebug->append("Status : OK - Action : " + port->getAction());
if(port->getAction()=="reset") // si l'action est un reset
{
    port->ecrire("ATRE\r");
    qDebug() << "commande : ATRE\r";
    port->setAction("save");
}
else
if(port->getAction()=="writeId") //si l'action est l'obtention du pan id
{
    const char *commande = ("ATID"+ui->lePAN->text()+"\r").toStdString().c_str(); // c_str permet de récupérer le char commande dans l'objet string

    qDebug() << "commande : " << commande; //affiche le pan id
    port->ecrire(commande);
    port->setAction("writeCe");
}
else
if(port->getAction()=="writeCe")
{
    QString Ce ="0";
    if(ui->rbCoordinateur->isChecked()) Ce ="1"; // si le radio button rbCoordinateur est sélectionné, définir le Ce à 1
    const char *commande = ("ATJN"+Ce+"\r").toStdString().c_str(); // toStdString convertie un objet QString en std::string
    qDebug() << "commande : " << commande; //affiche le paramètre Ce (définit à 1 pour le rôle du
    port->ecrire(commande);
    port->setAction("writeJv");
}
else
if(port->getAction()=="writeJv")
{
    QString Jv ="0";
    if(ui->rbRouteur->isChecked()) Jv ="1"; //si le radio button rbRouteur est sélectionné, définir le Jv à 1 (si à 1, le routeur va chercher la présence du coordinateur sur le canal)
    const char *commande = ("ATJV"+Jv+"\r").toStdString().c_str();
    qDebug() << "commande : " << commande;
    port->ecrire(commande);
    port->setAction("writeDh");
}
else
if(port->getAction()=="writeDh")
{
     const char *commande = "ATDH0\r";
     qDebug() << "commande : " << commande;
     port->ecrire(commande);
     port->setAction("writeDl");
}
else
if(port->getAction()=="writeDl") //adresse de destination minimale
{
    QString Dl ="0";
    if(ui->rbCoordinateur->isChecked()) Dl ="FFFF"; //si le radio buton rbCoordinateur est sélectionné, définit l à FFFF
    const char *commande = ("ATDL"+Dl+"\r").toStdString().c_str();
    qDebug() << "commande : " << commande;
    port->ecrire(commande);
    port->setAction("writeNi");
}
    else
    if(port->getAction()=="writeNi") // écrire le nom du module
    {
        const char *commande = ("ATNI"+ui->leNom->text()+"\r").toStdString().c_str();
        qDebug() << "commande : " << commande;
        port->ecrire(commande);
        port->setAction("save"); //sauvegarde la commande entrée
    }
    else
    if(port->getAction()=="save")
    {
        port->ecrire("ATWR\r");
        qDebug() << "commande : ATWR\r";
        port->setAction("readId");
    }
    else
        if(port->getAction()=="readId")
        {
            port->ecrire("ATID\r");
            qDebug() << "commande : ATID\r";
        }

if(port->getAction()=="exit")
{
    qDebug() << "Sortie du Mode conf : OK";
}

}


 void app_gtc::slotConfXbee(QByteArray conf)
 {
     if(port->getAction()=="readId")
     {
         qDebug() << "ID : " << QString(conf) ;
         ui->lePAN->setText(QString(conf));
         qDebug() << "commande : ATJN\r";
         port->ecrire("ATJN\r");
         port->setAction("readCe");
     }
     else
     if(port->getAction()=="readCe")
     {
         qDebug() << "CE : " << QString(conf) ;
         if(conf.at(0)=='1') ui->rbCoordinateur->setChecked(true);
         qDebug() << "commande : ATJV\r";
         port->ecrire("ATJV\r");
         port->setAction("readJv");
     }
     else
     if(port->getAction()=="readJv")
     {
         qDebug() << "JV : " << QString(conf) ;
         if(conf.at(0)=='1') ui->rbRouteur->setChecked(true);
         qDebug() << "commande : ATDH\r";
         port->ecrire("ATDH\r");
         port->setAction("readDh");
     }
     else
     if(port->getAction()=="readDl")
     {
         qDebug() << "DL : " << QString(conf) ;
         qDebug() << "commande : ATNI\r";
         port->ecrire("ATNI\r");
         port->setAction("readNi");
     }
     else
     if(port->getAction()=="readDl")
     {
             qDebug() << "DL : " << QString(conf) ;
             qDebug() << "commande : ATNI\r";
             port->ecrire("ATNI\r");
             port->setAction("readNi");
     }
     if(port->getAction()=="readNi") //Note Identifier
     {
         qDebug() << "NI : " << QString(conf) ;
         ui->leNom->setText(QString(conf));
         qDebug() << "commande : ATCN\r";
         port->ecrire("ATCN\r");
         port->setAction("exit");
     }
 }



 void app_gtc::on_pbConfigurer_clicked()
 {
     if(ui->leNom->text()=="" || ui->lePAN->text()=="")
     {
         qDebug() << "Erreur : PAN ou Nom module vide";
         ui->txtDebug->append("Erreur : PAN ou Nom module vide");

     }
     else
     {
         port->ecrire("+++");
         port->setAction("writeId");
         ui->grpConfig->setEnabled(false);
         //ui->pbModifier->setEnabled(true);
         //ui->pbConfigurer->setEnabled(false);
         ui->txtDebug->append("Configuration réussie");

     }
 }*/

 /*void app_gtc::on_pbModifier_clicked()
 {
     if(ui->leNom->text()=="" || ui->lePAN->text()=="")
     {
         qDebug() << "Erreur : PAN ou Nom module vide";
         ui->txtDebug->append("Erreur : PAN ou Nom module vide");

     }
     else
     {
         port->ecrire("+++");
         port->setAction("writeId");
         ui->grpConfig->setEnabled(true);
         ui->pbConfigurer->setEnabled(true);
         ui->pbModifier->setEnabled(false);

     }
 }*/

void app_gtc::on_pbRefresh_clicked()
{

}
