#include "ihmxbeeconf.h"
#include "ui_ihmxbeeconf.h"

IHMXbeeConf::IHMXbeeConf(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::IHMXbeeConf)
{
    ui->setupUi(this);

    QStringList listPort;
    QList<QSerialPortInfo> pdispo;
    pdispo = QSerialPortInfo::availablePorts();
    for (int i=0 ; i<pdispo.size() ; i++) {
        QString unPort = pdispo.at(i).portName();
        listPort.append(unPort);
    } // for
    ui->cbPort->addItems(listPort);
}

IHMXbeeConf::~IHMXbeeConf()
{
    delete ui;
}

void IHMXbeeConf::on_pbEnvoyer_clicked()
{
    QString msg;
    msg = ui->writeMsg->text();
    ui->txtDebug->append("message: " + msg);
    qDebug()<<"message: "<< msg;
}

void IHMXbeeConf::on_pbConnexion_clicked()
{
    if(ui->pbConnexion->text()=="C&onnexion") // si le texte du bouton est "Connexion"
    {
        QString nomPort = ui->cbPort->currentText();
        port = new Cserial(this, nomPort);
        port->initialiser(QSerialPort::Baud9600, QSerialPort::Data8,
                         QSerialPort::NoParity, QSerialPort::OneStop,
                         QSerialPort::NoFlowControl);
        connect(port, SIGNAL(sigOk()), this, SLOT(slotStatusOK())); // connexion du signal au slot
        connect(port, SIGNAL(sigConfXbee(QByteArray)), this, SLOT(slotConfXbee(QByteArray)));
        if(port->ouvrirPort())
        {
            ui->pbConnexion->setText("&Déconnexion");
            ui->txtDebug->append("Port " + nomPort + " ouvert!");
            qDebug()<<"Port"<<nomPort<<" ouvert !"; //affichage du nom du port ouvert dans le debugger
            ui->grpConfig->setEnabled(true);
            ui->grpDebug->setEnabled(true);
            ui->grpTrans->setEnabled(true);
            ui->pbConfigurer->setEnabled(true);
        }
        else
        {
        qDebug() << "Echec de l'ouverture du port " << nomPort << " !"; //si le port n'est pas ouvert, une erreur est affichée
        ui->grpConfig->setEnabled(false);
        ui->grpDebug->setEnabled(false);
        ui->grpTrans->setEnabled(false);
        ui->pbConfigurer->setEnabled(false);
    }
    }
    else
    {
         ui->grpConfig->setEnabled(false);
         ui->grpDebug->setEnabled(false);
         ui->grpTrans->setEnabled(false);
         ui->pbConfigurer->setEnabled(false);
         disconnect(port, SIGNAL(sigOk()), this, SLOT(slotStatusOK()));
         disconnect(port, SIGNAL(sigConfXbee(QByteArray)), this, SLOT(slotConfXbee(QByteArray)));
         delete port;
         qDebug() << "Déconnexion effectuée";
         ui->pbConnexion->setText("C&onnexion");
    }
}

int Cserial::ecrire(const char *trame)
{
    return m_serial->write(trame);
}

QString Cserial::getAction() const
{
    return m_action;
}

void Cserial::setAction(const QString &action)
{
    m_action = action;
}

void IHMXbeeConf::slotStatusOK()
{
qDebug() << "Status : OK - Action : " << port->getAction(); //affiche dans le debugger l'action effectuée
ui->txtDebug->append("Status : OK - Action : " + port->getAction());
if(port->getAction()=="reset") // si l'action est un reset
{
    port->ecrire("ATRE\r");
    qDebug() << "commande : ATRE\r";
    port->setAction("save");
}
else
if(port->getAction()=="writeId") //si l'action est l'obtention du pan id
{
    const char *commande = ("ATID"+ui->lePAN->text()+"\r").toStdString().c_str(); // c_str permet de récupérer le char commande dans l'objet string

    qDebug() << "commande : " << commande; //affiche le pan id
    port->ecrire(commande);
    port->setAction("writeCe");
}
else
if(port->getAction()=="writeCe")
{
    QString Ce ="0";
    if(ui->rbCoordinateur->isChecked()) Ce ="1"; // si le radio button rbCoordinateur est sélectionné, définir le Ce à 1
    const char *commande = ("ATJN"+Ce+"\r").toStdString().c_str(); // toStdString convertie un objet QString en std::string
    qDebug() << "commande : " << commande; //affiche le paramètre Ce (définit à 1 pour le rôle du
    port->ecrire(commande);
    port->setAction("writeJv");
}
else
if(port->getAction()=="writeJv")
{
    QString Jv ="0";
    if(ui->rbRouteur->isChecked()) Jv ="1"; //si le radio button rbRouteur est sélectionné, définir le Jv à 1 (si à 1, le routeur va chercher la présence du coordinateur sur le canal)
    const char *commande = ("ATJV"+Jv+"\r").toStdString().c_str();
    qDebug() << "commande : " << commande;
    port->ecrire(commande);
    port->setAction("writeDh");
}
else
if(port->getAction()=="writeDh")
{
     const char *commande = "ATDH0\r";
     qDebug() << "commande : " << commande;
     port->ecrire(commande);
     port->setAction("writeDl");
}
else
if(port->getAction()=="writeDl") //adresse de destination minimale
{
    QString Dl ="0";
    if(ui->rbCoordinateur->isChecked()) Dl ="FFFF"; //si le radio buton rbCoordinateur est sélectionné, définit l à FFFF
    const char *commande = ("ATDL"+Dl+"\r").toStdString().c_str();
    qDebug() << "commande : " << commande;
    port->ecrire(commande);
    port->setAction("writeNi");
}
    else
    if(port->getAction()=="writeNi") // écrire le nom du module
    {
        const char *commande = ("ATNI"+ui->leNom->text()+"\r").toStdString().c_str();
        qDebug() << "commande : " << commande;
        port->ecrire(commande);
        port->setAction("save"); //sauvegarde la commande entrée
    }
    else
    if(port->getAction()=="save")
    {
        port->ecrire("ATWR\r");
        qDebug() << "commande : ATWR\r";
        port->setAction("readId");
    }
    else
        if(port->getAction()=="readId")
        {
            port->ecrire("ATID\r");
            qDebug() << "commande : ATID\r";
        }

if(port->getAction()=="exit")
{
    qDebug() << "Sortie du Mode conf : OK";
}

}

void IHMXbeeConf::slotConfXbee(QByteArray conf)
{
    if(port->getAction()=="readId")
    {
        qDebug() << "ID : " << QString(conf) ;
        ui->lePAN->setText(QString(conf));
        qDebug() << "commande : ATJN\r";
        port->ecrire("ATJN\r");
        port->setAction("readCe");
    }
    else
    if(port->getAction()=="readCe")
    {
        qDebug() << "CE : " << QString(conf) ;
        if(conf.at(0)=='1') ui->rbCoordinateur->setChecked(true);
        qDebug() << "commande : ATJV\r";
        port->ecrire("ATJV\r");
        port->setAction("readJv");
    }
    else
    if(port->getAction()=="readJv")
    {
        qDebug() << "JV : " << QString(conf) ;
        if(conf.at(0)=='1') ui->rbRouteur->setChecked(true);
        qDebug() << "commande : ATDH\r";
        port->ecrire("ATDH\r");
        port->setAction("readDh");
    }
    else
    if(port->getAction()=="readDl")
    {
        qDebug() << "DL : " << QString(conf) ;
        qDebug() << "commande : ATNI\r";
        port->ecrire("ATNI\r");
        port->setAction("readNi");
    }
    else
    if(port->getAction()=="readDl")
    {
            qDebug() << "DL : " << QString(conf) ;
            qDebug() << "commande : ATNI\r";
            port->ecrire("ATNI\r");
            port->setAction("readNi");
    }
    if(port->getAction()=="readNi") //Note Identifier
    {
        qDebug() << "NI : " << QString(conf) ;
        ui->leNom->setText(QString(conf));
        qDebug() << "commande : ATCN\r";
        port->ecrire("ATCN\r");
        port->setAction("exit");
    }
}



void IHMXbeeConf::on_pbConfigurer_clicked()
{
    if(ui->leNom->text()=="" || ui->lePAN->text()=="")
    {
        qDebug() << "Erreur : PAN ou Nom module vide";
        ui->txtDebug->append("Erreur : PAN ou Nom module vide");

    }
    else
    {
        port->ecrire("+++");
        port->setAction("writeId");
        ui->grpConfig->setEnabled(false);
        ui->pbModifier->setEnabled(true);
        ui->pbConfigurer->setEnabled(false);
        ui->txtDebug->append("Configuration réussie");

    }
}

void IHMXbeeConf::on_pbModifier_clicked()
{
    if(ui->leNom->text()=="" || ui->lePAN->text()=="")
    {
        qDebug() << "Erreur : PAN ou Nom module vide";
        ui->txtDebug->append("Erreur : PAN ou Nom module vide");

    }
    else
    {
        port->ecrire("+++");
        port->setAction("writeId");
        ui->grpConfig->setEnabled(true);
        ui->pbConfigurer->setEnabled(true);
        ui->pbModifier->setEnabled(false);

    }
}
