// CSerial.cpp
#include "cserial.h"

CSerial::CSerial(QObject *parent, const QString &nomPort)
{

    m_parent = parent;
    m_Sp = new QSerialPort(parent);
    m_Sp->setPortName(nomPort);
    connect(m_Sp, SIGNAL(readyRead()), this, SLOT(onReadyRead())); //connection entre le signal readyRead et le slot onReadyRead
    connect(m_Sp, SIGNAL(error(QSerialPort::SerialPortError)), this, SLOT(onErreur(QSerialPort::SerialPortError)));
    qDebug() << "CSerial::L'objet m_Sp du port  " << nomPort << "est créé"; //affichage sur la console de la création du port

}

CSerial::~CSerial()
{
    m_Sp->close();
    delete m_Sp;
    qDebug() << "CSerial::m_Sp est détruit";    // affiche "CSerial::m_Sp est détruit"
}

int CSerial::initialiser(QSerialPort::BaudRate vitesse, QSerialPort::DataBits data,
                         QSerialPort::Parity parity, QSerialPort::StopBits nbStop,
                         QSerialPort::FlowControl flow)
{
    m_Sp->setBaudRate(vitesse); // le débit en bauds de la communication
    m_Sp->setDataBits(data); //  le nombre de bits de données
    m_Sp->setParity(parity); // le schéma de parité utilisé
    m_Sp->setStopBits(nbStop); // définit le nombre de bits d'arrêts utilisé
    m_Sp->setFlowControl(flow); // définit le contrôle de flux utlisé
    return 0;
}

int CSerial::ouvrirPort()
{
    bool res=false;
    res=m_Sp->open(QIODevice::ReadWrite);
    if (!res) {     // si le résultat ne s'ouvre pas
        m_Sp->close();
    } // if res
    return m_Sp->isOpen();
}

int CSerial::ecrire(const char *trame)
{
    int lg = m_Sp->write(trame,strlen(trame));
    if ( lg < strlen(trame)) {
        return ERREUR;
    } // if erreur
    //m_Sp->waitForBytesWritten(2000); // stop le thread actuel pour attendre max 2s écriture
    return lg;
}

void CSerial::onReadyRead()
{
    QByteArray recu;
    recu = m_Sp->read(100);
    mRecu.append(recu); //copie du signal reçu
    if(recu.endsWith('\n'))
    {
        emit sigData(mRecu); //emission du signal reçu
        mRecu.clear(); //efface le contenu du signal reçu
    }
}

void CSerial::onErreur(QSerialPort::SerialPortError err)
{
    emit sigErreur(err); //emission du signal sigErreur
}
