#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "cserial.h"
#include "../influxdb.hpp"
#include <iostream>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonValue>
#include <QDebug>
#include <QChar>
#include <QtGlobal>
#include <QString>

using namespace std;


 string resp;
 QByteArray commande;


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this); //initialise l'interface utilisateur

    ui->cbPorts->clear();
    ui->cbPorts->addItems(this->listerPorts()); //ajout des ports?

    timer = new QTimer();
    connect(timer, &QTimer::timeout,this ,&MainWindow::onTimerOut);
    timer->setInterval(1000);

    QIntValidator *testInt = new QIntValidator(this);
    testInt->setRange(10,120);

    QRegExp regex("(([0-1][0-9])|([2][0-3])):([0-5][0-9]):([0-5][0-9])");
    ui->gbConnexionXbee->setEnabled(false);

 }

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_pbConnecterXbee_clicked()
{
    if (ui->pbConnecterXbee->text() == "Connecter")
    {
        mXbee = new CSerial(this, "/dev/"+ui->cbPorts->currentText());
        connect(mXbee, SIGNAL(sigErreur(QSerialPort::SerialPortError)), this, SLOT(on_erreur(QSerialPort::SerialPortError)));
        connect(mXbee, SIGNAL(sigData(QByteArray)), this, SLOT(on_recevoirDataDuPeriph(QByteArray)));

        mXbee->initialiser(QSerialPort::Baud9600,
                           QSerialPort::Data8,
                           QSerialPort::NoParity,
                           QSerialPort::OneStop,
                           QSerialPort::NoFlowControl);
        if(mXbee->ouvrirPort())
        {
            ui->pbConnecterXbee->setText("Déconnecter"); //dès que le port est ouvert, afficher le texte "Déconnecter"
            qDebug() << "Port ouvert";
            ui->cbPorts->setEnabled(false);
            ui->pbRaffraichirListePorts->setEnabled(false);
            timer->start();
        }
        else
        {
            qDebug() << "Le Port n'est pas ouvert !"; //si le port n'est pas ouvert, affiche le message d'erreur
            ui->cbPorts->setEnabled(true);
            ui->pbRaffraichirListePorts->setEnabled(true);

        }
    }
    else
    {
        ui->pbConnecterXbee->setText("Connecter");
        ui->cbPorts->setEnabled(true);
        ui->pbRaffraichirListePorts->setEnabled(true);
        timer->stop();

        delete mXbee;
        QString messageFermer = mDate.currentDate().toString("dd/MM/yyyy") + " " + mNow.currentTime().toString("hh:mm:ss") + " : ";
        messageFermer += "Le port " + ui->cbPorts->currentText() + " est fermé";

    } // else
}

void MainWindow::on_erreur(QSerialPort::SerialPortError err)
{
    //ui->teDebug->append("Erreur voie série !");
    QString messageErreur = mDate.currentDate().toString("dd/MM/yyyy") + " " + mNow.currentTime().toString("hh:mm:ss") + " : ";
    qDebug() << "Serial Port : " << err;
    switch (err) {
    case QSerialPort::NoError:
        messageErreur += "Le port " + ui->cbPorts->currentText() + " est ouvert";
        break;
    case QSerialPort::DeviceNotFoundError:
        messageErreur += "Le port " + ui->cbPorts->currentText() + " n'est pas présent sur le système";
        break;
    case QSerialPort::PermissionError:
        messageErreur += "Permissions d'accéder au port " + ui->cbPorts->currentText() + " non accordées";
        break;
    case QSerialPort::OpenError:
        messageErreur += "Errur lors de l'ouvertur du port " + ui->cbPorts->currentText();
        break;
    case QSerialPort::NotOpenError:
        messageErreur += "Le port " + ui->cbPorts->currentText() + " est ouvert mais l'opération ne peut être exécutée";
        break;
    case QSerialPort::WriteError:
        messageErreur += "Impossible d'écrire sur le port " + ui->cbPorts->currentText();
        break;
    case QSerialPort::ReadError:
        messageErreur += "Impossible de lire le port " + ui->cbPorts->currentText();
        break;
    case QSerialPort::ResourceError:
        messageErreur += "Le port " + ui->cbPorts->currentText() + " est indisponible. Il a été retiré du système";
        break;

    default:
        messageErreur += "Erreur voie série !";
        break;

    }
    }
void MainWindow::on_recevoirDataDuPeriph(QByteArray txt)
{
    qDebug() << txt;
    QList<QByteArray> trame;
    trame = txt.split(SEPARATEUR);
    qDebug() << "Adresse : " << trame.at(0);
    qDebug() << "Ordre : " << trame.at(1);
    qDebug() << "Data : " << trame.at(2);


    if(trame.at(1)=="AMP")
    {
        mAmpere = trame.at(2).toFloat();
        mWatt = mAmpere*230/1000;
        QString sAmpere, sWatt;
       }
}
bool MainWindow::mXbeeConnected()
{
    cout << "Sombra connectée" << endl;
    if (envoyerCommande()==1)
    {
        cout << "FUN.exe" << endl;
        return true;
    }
    else {
        cout << "NOP.exe" << endl;
        return false;
    }
}


char MainWindow::calculChecksum(QByteArray message)
{
    char cs = 0;
    for (int i = 0; i < message.length(); i++)
    {
        cs = cs ^ message.at(i);
    }

    qDebug() << "Checksum = " << cs;
    return cs;
}

//void MainWindow::on_rbAutomatique_toggled(bool checked)
//{
//    if(checked) mData = "AUTO";
//    else mData = "MANUAL";
//    mOrdre = "MOD";
//    envoyerCommande();

//}


//void MainWindow::on_cbEST_stateChanged(int arg1)
//{
//    mOrdre = "EST";
//    if(!arg1)
//    {
//        mData = "False";
//    }
//    else mData = "True";
//    envoyerCommande();

//}

/*char MainWindow::calculChecksum(QByteArray message)
{
    char check = 0;
    for (int i = 0; i < message.length(); i++)
    {
        check= check ^message.at(i);
    }
    qDebug() << "Checksum :" << check;
    return check;
}*/

int MainWindow::envoyerCommande()
{

    commande.append(mAdresse);
    commande.append(SEPARATEUR);
    commande.append(/*mOrdre*/"DIM");
    commande.append(SEPARATEUR);
    commande.append(mData);
    commande.append(SEPARATEUR);
    QChar cs = calculChecksum(commande);
    QString csStr;
    csStr.setNum(cs.unicode(),16);
    commande.append(csStr);
    commande.append("\n");

    if(mXbee->ecrire(commande.toStdString().c_str()))
    {
        qDebug() << "Data transmisent : " << commande;

        influxdb_cpp::server_info si_new("192.168.4.34", 8086, "bddgtc", "", "");
/*
        double mDoubleTime = mTime.toDouble();
        double mDoubleTimeMin = mDoubleTime-10000;
        double mDoubleTimeMax = mDoubleTime+10000;
        long long mLongTime = mDoubleTime;
        long long mLongTimeMin = mDoubleTimeMin;
        long long mLongTimeMax = mDoubleTimeMax;
        string mStringTime = mTime.toStdString();
        string mStringTimeMin = to_string(mLongTimeMin);
        string mStringTimeMax = to_string(mLongTimeMax);
        char * mCharTimeMin = reinterpret_cast<char *>(&mLongTimeMin);
        char * mCharTimeMax = reinterpret_cast<char *>(&mLongTimeMax);

        cout << "mLongTime: " << mLongTime << endl;
        cout << mStringTimeMax << endl;
        cout << mStringTimeMin << endl;
*/

        string mStringAdresse = mAdresse.toStdString();
        int mIntAdresse = mAdresse.toInt();
        string mStringData = mData.toStdString();
        //string mStringTime = mTime.toStdString();

        float mFloatData = mData.toFloat();
       //influxdb_cpp::server_info si_new("192.168.4.34", 8086, "bddgtc&epoch=ns", "", "");
       // influxdb_cpp::query(resp, "INSERT dim,add="+ mStringAdresse +",flag=1 value="+ mStringData +" "+ mStringTime +"", si_new);
       // cout << resp << endl;
        int ret = influxdb_cpp::builder()
            .meas("dim")
            .tag("flag", "1")
            .tag("add",mStringAdresse)
            .field("value", mFloatData)
            //.timestamp(mTime)
            .post_http(si_new, &resp);
        cout << ret << endl << resp << endl;
        cout << "insert fait" << endl;

        string query = "DELETE FROM dim WHERE flag='0' AND add = '"+ mStringAdresse +"' AND time="+ mTime +"";
       cout << "query: " << query << endl;
        influxdb_cpp::query(resp,query ,si_new);
        cout << resp << endl;


        commande.clear();
        return 1;
    }
    commande.clear();

    return 0;
}

QStringList MainWindow::listerPorts()
{

    QStringList stringList;
    QList<QSerialPortInfo> spi;
    spi = QSerialPortInfo::availablePorts();
    if(!spi.empty())
    {
        for (int i=0 ; i<spi.size() ; i++)
        {
            QString str = spi.at(i).portName();
            stringList.append(str);
        } // for
    }
    return stringList;
}

void MainWindow::onTimerOut(){  // Exécuté par le timer

    // https://github.com/orca-zhang/influxdb-c

        influxdb_cpp::server_info si_new("192.168.4.34", 8086, "bddgtc&epoch=ns", "", "");
        influxdb_cpp::query(resp, "SELECT value, add FROM dim WHERE flag='0'", si_new);
        //m_respGetted=resp;
        //cout <<m_respGetted.length()<<endl;
        //cout <<m_respGetted.<<endl;


       char * str = (char *) malloc(resp.length() + 1); // +1 pour le '\0'
       strcpy(str, resp.c_str());

        QJsonDocument json = QJsonDocument::fromJson(QByteArray(resp.c_str()));
        QJsonObject jo = json.object();
        QJsonArray results = jo["results"].toArray();
        QJsonValue jv = results[0];
        QJsonObject jo2 = jv.toObject();
        QJsonArray series = jo2["series"].toArray();
        QJsonObject jo3 = series[0].toObject();
        QJsonArray values = (jo3["values"].toArray())[0].toArray();

//        QVariantMap json_map = jo3.toVariantMap();
//        cout << "time : " << json_map["time"].toString().toStdString() << endl;

          cout << "resp : " << resp << endl;
          //cout << "time  : " << values[0].toDouble() << endl;
          //cout << "value : " << values[1].toInt() << endl;
          //cout << "add   : " << values[2].toString().toStdString() << endl;


        if(values[1].toInt() != NULL)
        {
            cout << values[1].toInt() << endl;
            mData = QString::number(values[1].toInt()).toLatin1();
            mAdresse = values[2].toString().toLatin1();
            mTime = resp.substr(98,19)/*.toDouble()).toLatin1()*/;

            cout << "time : " << mTime << endl;
            cout << "mData : " << QString(mData).toStdString() << endl;
            cout << "mAdresse : " << QString(mAdresse).toStdString() << endl;

            // mXbeeConnected();
            envoyerCommande();
        }
        else
        {
            cout << "pas de donnée" << endl;
        }
        //cout << "commande : " << QString(commande).toStdString() << endl;
}

/*
 *
 * {"results":
 *      [{"statement_id":0,"series":
 *          [{"name":"dim","columns":
 *              ["time","value","add"],"values":
 *              [["2019-05-17T12:59:50.9322559Z",50,"2428"]
 *              ]                              +
 *          }]
 *      }]
 * }
 *
 * qjson.results[0].series[0].values[2] => 50
 *
 * */






void MainWindow::on_pbMdp_clicked()
{
    if (ui->leMdp->text().toStdString() == mdp && ui->pbMdp->text().toStdString() == "Confirmer")
    {
        ui->gbConnexionXbee->setEnabled(true);
        ui->leMdp->setText("");
        ui->pbMdp->setText("Verrouiller");
    }
    else
    {
        ui->leMdp->setText("");
        ui->pbMdp->setText("Confirmer");
        ui->gbConnexionXbee->setEnabled(false);
    }
}
